#include <fstream>
#include <iostream>
#include <vector>
#include <list>
#include <deque>
#include <cmath>
#include <algorithm>

void createDataStream(const std::string& filename) {
    std::ofstream file(filename);
    for (int i = 0; i < 100; i++)
        file << -50 + rand() % 101 << " ";
}

void createDataGenerate(const std::string& filename) {
    std::ofstream file(filename);
    std::vector<int> data(100);
    std::generate(data.begin(), data.end(), [] {return -50 + std::rand() % 101;});
    std::copy(data.begin(), data.end(), std::ostream_iterator<int>(file, " "));
    file << std::endl;
}

template <typename Cont>
Cont loadDataStream(const std::string& path) {
    std::ifstream file(path);
    Cont cont;
    using value_type = typename Cont::value_type;
    value_type value;
    while (!file.eof()) {
        file >> value;
        cont.push_back(value);
    }
    return cont;
}

template <typename Cont>
Cont loadDataIterator(const std::string& path) {
    std::ifstream file(path);
    Cont cont(100);
    using value_type = typename Cont::value_type;
    std::copy(std::istream_iterator<value_type>(file), std::istream_iterator<value_type>(), cont.begin());
    return cont;
}

template <typename Container>
void modifyTemplate(Container& src) {
    Container result;
    auto max = *std::max_element(src.begin(), src.end());
    auto last = *std::prev(src.end());
    for (auto& el: src) {
        el = el + std::sqrt(max * last);
    }
}

template <typename SrcIter, typename DestIter>
void modifyIter(const SrcIter src_begin, const SrcIter src_end, DestIter dest_begin) {
    auto max = *std::max_element(src_begin, src_end);
    auto last = *std::prev(src_end);
    auto result_begin = dest_begin;
    for (auto it = src_begin; it != src_end; ++it)
        *(result_begin++) = *it + std::sqrt(last * max);
}

template <typename Cont>
void modifyForEach(Cont& src) {
    auto max = *std::max_element(src.begin(), src.end());
    auto last = *std::prev(src.end());
    auto modify = [last, max](auto& el) {
        el = el + std::sqrt(last * max);
        return el;
    };
    std::for_each(src.begin(), src.end(), modify);
}

void modifyTransform(const std::string& in_fname, const std::string& out_fname) {
    std::ifstream fin(in_fname);
    std::ofstream fout(out_fname);
    std::vector<float> vec(100);
    std::copy(std::istream_iterator<float>(fin), std::istream_iterator<float>(), vec.begin());
    auto max = *std::max_element(vec.begin(), vec.end());
    auto last = *std::prev(vec.end());
    auto modify = [last, max](auto& el) {
        el = el + std::sqrt(last * max);
        return el;
    };
    std::transform(vec.begin(), vec.end(), std::ostream_iterator<float>(fout, " "), modify);
    fout << std::endl;
}

template <typename Cont>
void outputResultStream(const std::string& filename, const Cont& src) {
    std::ofstream fout(filename);
    for (const auto& el : src)
        fout << el << " ";
    fout << std::endl;
}

template <typename Cont>
void outputResultCopy(const std::string& filename, const Cont& src) {
    using value_type = typename Cont::value_type;
    std::ofstream fout(filename, std::ios::app);
    std::copy(src.begin(), src.end(), std::ostream_iterator<value_type>(fout, " "));
    fout << std::endl;
}

template <typename Cont>
void printContainer(const Cont& src)
{
    for (auto it = src.cbegin(); it != src.cend(); ++it)
        std::cout << *it << " ";
}
